@@ -15,7 +15,7 @@ if 'CFG_INVENIO_SRCDIR' in os.environ:
 elif 'CFG_INVENIO_SRC' in os.environ:
     SRC_PATH = [os.environ['CFG_INVENIO_SRC']]
 else:
-    SRC_PATH = [ os.path.expanduser("~/src/invenio"), ]
+    SRC_PATH = [os.path.expanduser("~/src/invenio"), ]
 
 
 # Try if we're in a virtualenv or CFG_INVENIO_PREFIX is set.
@@ -52,9 +52,14 @@ STATIC_FILES = {
 CONFIG_FILENAME = 'invenio.conf'
 LOCAL_CONFIG_FILENAME = 'invenio-local.conf'
 
-# Max time we have available to process a request
+# Print a warning after xx seconds when processing a request
 REQUEST_TIMEOUT = 60
 
+# Spawns a server that will accept connections to be able to
+# run arbitrary python code on the work
+USE_CONSOLE = False
+
+
 try:
     import config_local
     for setting in dir(config_local):
@@ -62,4 +67,3 @@ try:
             globals()[setting] = getattr(config_local, setting)
 except ImportError:
     pass
-
