@@ -5,6 +5,7 @@ import signal
 import optparse
 import socket
 import traceback
+import logging
 from itertools import chain
 
 try:
@@ -128,9 +129,10 @@ def reloader_loop(files, reloader=None, interval=1):
                     _log('info', ' * Detected change in %r' % filename)
                 has_changes = True
 
-
         if has_changes and reloader:
             reloader()
+            if not reloader.worker_pid:
+                return
             time.sleep(1)
 
         time.sleep(interval)
@@ -155,12 +157,11 @@ class Reloader(object):
         """Called to reload the worker"""
         if self.worker_pid:
             kill_worker(self.worker_pid)
-        try:
-            self.worker_pid = spawn_server(self.options, self.server_socket)
-        except:
-            # Set to None in case, we kill the whole process group
-            self.worker_pid = None
-            raise
+        self.worker_pid = spawn_server(self.options, self.server_socket)
+
+
+def print_traceback(sig, frame):
+    traceback.print_stack(frame)
 
 
 def start_server(options, server_socket, static_files=config.STATIC_FILES):
@@ -175,9 +176,16 @@ def start_server(options, server_socket, static_files=config.STATIC_FILES):
     wsgi.replace_error_handler()
     wsgi.wrap_warn()
 
+    signal.signal(signal.SIGUSR1, print_traceback)
+
+    # Hook debugging console
+    if config.USE_CONSOLE or options.use_console:
+        from rfoo.utils import rconsole
+        rconsole.spawn_server()
+
     static = dict([(k, config.INSTALL_PATH + v) for k, v in static_files.items()])
 
-    wsgi_app = partial(wsgi.application, options)
+    wsgi_app = wsgi.application
 
     if options.use_pdb:
         import pdb
@@ -215,7 +223,14 @@ def spawn_server(options, server_socket):
     _log('info', ' * Spawning worker')
     pid = os.fork()
     if pid == 0:
-        start_server(options, server_socket)
+        try:
+            start_server(options, server_socket)
+        except:
+            print traceback.format_exc()[:-1]
+            _log('info', ' * Worker crashed')
+            # We do not want to free this pid because it will be killed
+            while True:
+                time.sleep(6000)
     return pid
 
 
@@ -234,9 +249,6 @@ def parse_cli_options():
     parser.add_option('--no-http', action='store_false', dest='serve_http',
                       default=True, help='Disable http server, only update ' \
                       'invenio install')
-    parser.add_option('--buffer-output', dest='buffer_output', default=False,
-                      action='store_true', help='Buffer output to display\n' \
-                      'to display debug pages')
     parser.add_option('--pdb', dest='use_pdb', default=False,
                       action='store_true', help='Drop to python debugger\n' \
                       'on errors')
@@ -244,6 +256,8 @@ def parse_cli_options():
                       default=[], help='Source folder (one or more)')
     parser.add_option('-o', dest='install_path', metavar='INSTALL_PATH',
                       default=[], help='Path to Invenio installation.')
+    parser.add_option('--use-console', action='store_true', dest='use_console',
+                      default=False, help='Ability to open a remote console on worker')
     return parser.parse_args()
 
 
@@ -275,7 +289,7 @@ def bind_socket(options, ssl_context=None):
     return server_socket
 
 
-def prepare_server(options, ssl_context=None):
+def start_reloading_server(options, ssl_context=None):
     """Prepare http server
 
     First binds the socket to accept connections from,
@@ -293,23 +307,30 @@ def prepare_server(options, ssl_context=None):
         reloader()
         if reloader.worker_pid:
             invenio_files = list(generate_invenio_files_list())
+            # Our infinite loop is here
             reloader_loop(invenio_files, reloader)
-    except BaseException, e:
+    except BaseException:
         if reloader and reloader.worker_pid:
             kill_worker(reloader.worker_pid)
-        else:
-            # We are killing ourselves
-            # The python interpreter will not get a chance to print
-            # the traceback
-            if not isinstance(e, KeyboardInterrupt) \
-                                            and not isinstance(e, SystemExit):
-                print traceback.format_exc()[:-1]
-            os.killpg(0, signal.SIGKILL)
         raise
 
 
+def setup_logging():
+    logger = logging.getLogger()
+    logger.setLevel(logging.WARNING)
+    handler = logging.StreamHandler()
+    logger.addHandler(handler)
+
+    logger = logging.getLogger('werkzeug')
+    logger.setLevel(logging.INFO)
+    handler = logging.StreamHandler()
+    logger.addHandler(handler)
+    logger.propagate = False
+
+
 def _main():
     """Script entrance"""
+    setup_logging()
     (options, args) = parse_cli_options()
 
     # Override config SRC_PATH and INSTALL_PATH
@@ -320,7 +341,7 @@ def _main():
 
     if options.serve_http and options.auto_reload:
         print 'HTTP Server mode with reload mode'
-        prepare_server(options)
+        start_reloading_server(options)
     elif options.serve_http:
         print 'Simple HTTP Server mode'
         start_server(options, bind_socket(options))
@@ -338,4 +359,4 @@ def main():
 
 
 if __name__ == '__main__':
-    main()
\ No newline at end of file
+    main()
